
# q-expansion of newform 9282.2.a.e, downloaded from the LMFDB on 11 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 9282
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[-1], [-1], [0], [1], [2], [-1], [-1], [-5], [-2], [-10], [1], [0], [5], [-5], [-7], [3], [8], [-8], [4], [7], [14], [-3], [-10], [14], [-2], [-6], [13], [13], [10], [15], [2], [-6], [-3], [-3], [14], [-5], [1], [-3], [22], [11], [-16], [2], [5], [-10], [6], [-6], [0], [-4], [9], [2], [14], [-6], [-2], [15], [-21], [28], [-15], [20], [-10], [-9], [-11], [29], [4], [33], [19], [-17], [10], [2], [23], [-26], [-4], [-4], [-6], [4], [-37], [-16], [22], [35], [28], [22], [-30], [7], [36], [-16], [12], [-12], [-24], [-38], [26], [-16], [27], [12], [-12], [0], [-20], [37], [-15], [-28], [36], [-10], [-18], [-30], [28], [28], [-6], [-5], [4], [-4], [-16], [-9], [8], [-9], [6], [20], [16], [2], [-36], [30], [-26], [-30], [-20], [26], [38], [24], [34], [9], [8], [24], [0], [20], [26], [-1], [-31], [-32], [2], [-1], [6], [36], [-18], [-29], [24], [6], [27], [20], [-19], [12], [19], [30], [50], [-10], [-8], [-8], [51], [40], [-28], [-42], [24], [-57], [-36], [-50], [26], [34], [-12], [44], [-5], [-42], [24], [12], [50], [26], [33], [20], [-22], [18], [-56], [50], [1], [-33], [-61], [35], [23], [-6], [-43], [-12], [44], [-30], [7], [4], [-13], [22], [-3], [-6], [-13], [11], [-24], [-42], [33], [14], [33], [22], [59], [22], [30], [12], [-70], [-22], [-20], [58], [30], [50], [28], [40], [16], [4], [24], [-44], [-18], [21], [36], [-38], [0], [5], [25], [14], [17], [-64], [8], [46], [-5], [-30], [-4], [-73], [-40], [-16], [-50], [33], [-4], [49], [-75], [42], [-28], [-2], [21], [-7], [6], [36], [9], [-36], [-14], [-13], [-55], [-66], [-42], [39], [-30], [-21], [-25], [-24], [-57], [-21], [1], [48], [-50], [70], [-38], [56], [-6], [38], [0], [66], [66], [-34], [29], [34], [18], [38], [35], [-30], [12], [31], [50], [20], [-39], [22], [66], [57], [-34], [-75], [-30], [-61], [-36], [54], [-13], [-12], [-37], [-55], [5], [-18], [30], [29], [34], [-68], [-39], [-76], [-4], [33], [28], [-76], [-66], [56], [-32], [42], [-57], [-13], [72], [-2], [-34], [-41], [28], [-22], [-73], [-23], [-18], [-8], [44], [-62], [31], [42], [16], [38], [68], [48], [30], [36], [13], [47], [-45], [-30], [2], [64], [-86], [-62], [-24], [-76], [-50], [9], [-50], [60], [-40], [86], [-49], [18], [78], [26], [80], [-33], [59], [36], [33], [58], [-46], [-20], [-25], [-36], [36], [33], [-84], [45], [-68], [30], [19], [77], [-77], [37], [-11], [-91], [12], [-70], [94], [42], [48], [-29], [28], [-72], [42], [94], [79], [-27], [-76], [46], [-85], [-36], [45], [8], [-30], [84], [34], [75], [-26], [-12], [1], [-18], [-53], [-7], [9], [-66], [18], [41], [49], [-1], [25], [1], [78], [34], [-51], [-3], [17], [-48], [-10], [22], [102], [101], [36], [-21], [-54], [22], [-20], [6], [-5], [-81], [50], [-11], [-70], [-98], [-19], [1], [-67], [47], [26], [16], [-60], [-98], [41], [39], [6], [52], [62], [-96], [94], [-58], [16], [-14], [86], [60], [-12], [37], [-55], [-7], [40], [-86], [26], [-20], [-98], [83], [60], [46], [46], [22], [-79], [22], [56], [-43], [22], [91], [8], [-85], [-90], [98], [48], [-10], [110], [-17], [62], [21], [29], [-78], [-18], [56], [-70], [52], [54], [-30], [-43], [-73], [-101], [73], [-76], [-70], [50], [43], [-51], [-82], [-38], [-32], [116], [-11], [24], [-50], [40], [17], [-16], [15], [-10], [94], [11], [-8], [-49], [106], [33], [59], [120], [-54], [48], [108], [86], [-48], [-10], [-55], [-44], [60], [74], [115], [-16], [-36], [-18], [64], [-7], [-73], [-52], [-68], [21], [108], [-94], [-3], [-120], [-16], [-26], [-20], [55], [-32], [27], [102], [51], [75], [16], [-3], [-67]]
